Package: SpotSweeper
Title: Spatially-aware quality control for spatial transcriptomics
Version: 1.9.0
Date: 2025-10-14
Authors@R: c(
    person("Michael", "Totty", ,"mictott@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-9292-8556")),
    person("Stephanie", "Hicks", ,email = "shicks19@jhu.edu", role = c("aut"),
           comment = c(ORCID = "0000-0002-7858-0231")),
    person("Boyi", "Guo", ,email = "boyi.guo.work@gmail.com", role = c("aut"),
           comment = c(ORCID = "0000-0003-2950-2349")))
Description: Spatially-aware quality control (QC) software for both spot-level 
    and artifact-level QC in spot-based spatial transcripomics, such as 10x 
    Visium. These methods calculate local (nearest-neighbors) mean and variance 
    of standard QC metrics (library size, unique genes, and mitochondrial 
    percentage) to identify outliers spot and large technical artifacts.
License: MIT + file LICENSE
URL: https://github.com/MicTott/SpotSweeper
BugReports: https://support.bioconductor.org/tag/SpotSweeper
biocViews: Software, Spatial, Transcriptomics, QualityControl,
        GeneExpression,
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.1
Depends: R (>= 4.4.0)
Imports: SpatialExperiment, SummarizedExperiment, BiocNeighbors,
        SingleCellExperiment, stats, escheR, MASS, ggplot2, spatialEco,
        grDevices, BiocParallel
Suggests: knitr, BiocStyle, rmarkdown, scuttle, STexampleData, ggpubr,
        testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
LazyData: False
News: NEWS.md
git_url: https://git.bioconductor.org/packages/SpotSweeper
git_branch: devel
git_last_commit: 5c89af8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: no
Packaged: 2026-05-18 23:35:06 UTC; biocbuild
Author: Michael Totty [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9292-8556>),
  Stephanie Hicks [aut] (ORCID: <https://orcid.org/0000-0002-7858-0231>),
  Boyi Guo [aut] (ORCID: <https://orcid.org/0000-0003-2950-2349>)
Maintainer: Michael Totty <mictott@gmail.com>
Built: R 4.6.0; ; 2026-05-19 05:11:03 UTC; unix

# Generated by roxygen2: do not edit by hand

export(findArtifacts)
export(flagVisiumOutliers)
export(localOutliers)
export(localVariance)
export(plotQCmetrics)
export(plotQCpdf)
import(SingleCellExperiment)
import(SpatialExperiment)
importFrom(BiocNeighbors,findKNN)
importFrom(BiocParallel,MulticoreParam)
importFrom(MASS,rlm)
importFrom(SummarizedExperiment,colData)
importFrom(escheR,add_fill)
importFrom(escheR,add_ground)
importFrom(escheR,make_escheR)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_y_reverse)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(spatialEco,outliers)
importFrom(stats,kmeans)
importFrom(stats,prcomp)
importFrom(stats,resid)
importFrom(stats,var)
importFrom(utils,data)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# SpotSweeper <img src="man/figures/hexsticker.png" align="right" style="width:200px;" />

<!-- badges: start -->

[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![codecov](https://codecov.io/gh/MicTott/SpotSweeper/graph/badge.svg?token=M1MUJN7VS6)](https://codecov.io/gh/MicTott/SpotSweeper)
<!-- badges: end -->

`SpotSweeper` is a package developed for spatially-aware quality control
(QC) methods for the detection, visualization, and removal of both local
outliers and regional artifacts in spot-based spatial transcriptomics
data, such as 10x Genomics `Visium`, using standard QC metrics.

If you experience any issues using the package or would like to make a
suggestion, please open an issue on the [GitHub
repository](https://github.com/MicTott/SpotSweeper/issues).

To find more information, please visit the [documentation
website](http://MicTott.github.io/SpotSweeper).

<img src="./man/figures/schematic.png" width="100%" />

## Installation instructions

You can install the latest version of `SpotSweeper` from Bioconductor
with the following code:

``` r
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("SpotSweeper")
```

The latest development version can be installed from
[GitHub](https://github.com/MicTott/SpotSweeper) using the following:

``` r
if (!require("devtools")) install.packages("devtools")
remotes::install_github("MicTott/SpotSweeper")
```

## Tutorials

A detailed tutorial is available in the package vignette from
Bioconductor. A direct link to the tutorial / package vignette is
available
[here](https://mictott.github.io/SpotSweeper/articles/getting_started.html).

## Development tools

- Continuous code testing is possible thanks to [GitHub
  actions](https://www.tidyverse.org/blog/2020/04/usethis-1-6-0/)
  through `BiocStyle::Biocpkg('biocthis')`.
- The [documentation website](http://MicTott.github.io/SpotSweeper) is
  automatically updated thanks to `BiocStyle::CRANpkg('pkgdown')`.
- The code is styled automatically thanks to
  `BiocStyle::CRANpkg('styler')`.
- The documentation is formatted thanks to
  `BiocStyle::CRANpkg('devtools')` and `BiocStyle::CRANpkg('roxygen2')`.

This package was developed using `BiocStyle::Biocpkg('biocthis')`.

## ----style, echo = FALSE, results = 'asis'------------------------------------
BiocStyle::markdown()

## ----'install_dev', eval = FALSE----------------------------------------------
# if (!require("devtools")) install.packages("devtools")
# remotes::install_github("MicTott/SpotSweeper")

## ----'install', eval = FALSE--------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#     install.packages("BiocManager")
# }
# 
# BiocManager::install("SpotSweeper")

## ----example_spe--------------------------------------------------------------
library(SpotSweeper)

# load  Maynard et al DLPFC daatset
spe <- STexampleData::Visium_humanDLPFC()

# show column data before SpotSweeper
colnames(colData(spe))

# drop out-of-tissue spots
spe <- spe[, spe$in_tissue == 1]

## ----example_scuttle----------------------------------------------------------
# change from gene id to gene names
rownames(spe) <- rowData(spe)$gene_name

# identifying the mitochondrial transcripts
is.mito <- rownames(spe)[grepl("^MT-", rownames(spe))]

# calculating QC metrics for each spot using scuttle
spe <- scuttle::addPerCellQCMetrics(spe, subsets = list(Mito = is.mito))
colnames(colData(spe))

## ----example_local_outliers---------------------------------------------------
# library size
spe <- localOutliers(spe,
    metric = "sum",
    direction = "lower",
    log = TRUE
)

# unique genes
spe <- localOutliers(spe,
    metric = "detected",
    direction = "lower",
    log = TRUE
)

# mitochondrial percent
spe <- localOutliers(spe,
    metric = "subsets_Mito_percent",
    direction = "higher",
    log = FALSE
)

## ----example_combine_local_outliers-------------------------------------------
# combine all outliers into "local_outliers" column
spe$local_outliers <- as.logical(spe$sum_outliers) |
    as.logical(spe$detected_outliers) |
    as.logical(spe$subsets_Mito_percent_outliers)

## ----local_outlier_plot, fig.width=3, fig.height=3, dpi=100-------------------
library(escheR)


# all local outliers
plotQCmetrics(spe, metric = "sum_log", outliers = "local_outliers", point_size = 1.1, 
       stroke = 0.75) +
      ggtitle("All Local Outliers")


## ----example_artifactRemoval--------------------------------------------------
# load in DLPFC sample with hangnail artifact
data(DLPFC_artifact)
spe <- DLPFC_artifact

# inspect colData before artifact detection
colnames(colData(spe))

## ----artifact_QC_plots, fig.width=4, fig.height=4, dpi=100--------------------

plotQCmetrics(spe,
    metric = "expr_chrM_ratio",
    outliers = NULL, point_size = 1.1
) +
    ggtitle("Mitochondrial Percent")


## ----artifact_plot------------------------------------------------------------
# find artifacts using SpotSweeper
spe <- findArtifacts(spe,
    mito_percent = "expr_chrM_ratio",
    mito_sum = "expr_chrM",
    n_order = 5,
    name = "artifact"
)

# check that "artifact" is now in colData
colnames(colData(spe))

## ----artifact_visualization, fig.width=4, fig.height=4, dpi=100---------------
plotQCmetrics(spe,
    metric = "expr_chrM_ratio",
    outliers = "artifact", point_size = 1.1
) +
    ggtitle("Hangnail artifact")

## -----------------------------------------------------------------------------
utils::sessionInfo()

#' Plot QC metrics for a Single Sample in a SpatialExperiment object
#'
#' This function generates a plot for a specified sample within a
#' SpatialExperiment object, highlighting outliers based on a specified
#' metric. The plot visualizes the metric of interest and indicates outliers
#' with a distinct color.
#'
#' @param spe A SpatialExperiment object containing the data to be plotted.
#' @param sample_id A character string specifying the column name in
#'   `colData(spe)` that contains unique sample identifiers.
#'   Default is "sample_id".
#' @param sample A character string or numeric value specifying the sample to be
#'   plotted. By default, it plots the first unique sample found in
#'   `spe$sample_id`.
#' @param metric A character string specifying the metric to be visualized
#'   in the plot. This metric should be a column name in `colData(spe)`.
#' @param outliers A character string specifying the column name in
#'   `colData(spe)` that indicates whether a data point is considered an
#'   outlier. Default is NULL.
#' @param colors A character vector specifying the colors to be used for the
#'   gradient scale. If length is 2, the gradient will be a single color
#'   gradient.
#' @param stroke A numeric value specifying the border thickness for outlier
#'   points. Default is 1.
#' @param point_size A numeric value specifying the size of the points in the
#'  plot. Default is 2.
#'
#' @return The function returns a plot object created by `make_escheR` and
#'   modified with additional layers for visualizing the specified metric and
#'   outliers. The plot is not explicitly printed by the function and should be
#'   printed by the caller.
#'
#' @importFrom escheR make_escheR add_fill add_ground
#' @importFrom ggplot2 ggtitle scale_fill_gradient scale_y_reverse
#' @importFrom ggplot2 scale_color_manual scale_fill_gradient2 coord_fixed
#'
#'
#' @examples
#' library(SpotSweeper)
#' library(SpatialExperiment)
#' library(escheR)
#'
#' # load example data
#' spe <- STexampleData::Visium_humanDLPFC()
#'
#' # change from gene id to gene names
#' rownames(spe) <- rowData(spe)$gene_name
#'
#' # drop out-of-tissue spots
#' spe <- spe[, spe$in_tissue == 1]
#' spe <- spe[, !is.na(spe$ground_truth)]
#'
#' # Identifying the mitochondrial transcripts in our SpatialExperiment.
#' is.mito <- rownames(spe)[grepl("^MT-", rownames(spe))]
#'
#' # Calculating QC metrics for each spot using scuttle
#' spe <- scuttle::addPerCellQCMetrics(spe, subsets = list(Mito = is.mito))
#' colnames(colData(spe))
#'
#' # Identifying local outliers using SpotSweeper
#' spe <- localOutliers(spe,
#'                      metric = "sum",
#'                      direction = "lower",
#'                      log = TRUE
#' )
#'
#' plotQCmetrics(spe, metric="sum", outliers="sum_outliers")
#'
#' @export

plotQCmetrics <- function(
    spe, sample_id = "sample_id",
    sample = unique(spe$sample_id)[1], metric = "detected",
    outliers = NULL, point_size = 2,
    colors = c("white", "black"), stroke = 1) {
  # Subset the data to the specified sample
  spe.subset <- spe[, colData(spe)[[sample_id]] == sample]

  # Start building the plot
  p <- make_escheR(spe.subset) |>
    add_fill(var = metric, point_size = point_size)

  # Conditionally add outliers if they are not NULL
  if (!is.null(outliers)) {
    p <- p |> add_ground(
      var = outliers,
      stroke = stroke,
      point_size = point_size
    )
  }

  # Add title to the plot
  p <- p + ggtitle(paste0("Sample: ", sample))

  # Remove and replace scales to avoid warnings for re-coloring
  p$scales$scales <- list()

  # Conditionally add color scales based on the length of 'colors'
  if (length(colors) == 2) {
    p <- p + scale_fill_gradient(low = colors[1], high = colors[2]) +
      scale_y_reverse()

    # If outliers are not NULL, add a manual color scale for them
    if (!is.null(outliers)) {
      p <- p + scale_color_manual(
        name = "", # turn off legend name for ground
        values = c("TRUE" = "red", "FALSE" = "transparent")
      )
    }
  } else if (length(colors) > 2) {
    p <- p + scale_fill_gradient2(
      low = colors[1],
      mid = colors[2],
      high = colors[3]
    ) +
      scale_y_reverse()

    # If outliers are not NULL, add a manual color scale for them
    if (!is.null(outliers)) {
      p <- p + scale_color_manual(
        name = "", # turn off legend name for ground
        values = c("TRUE" = "red", "FALSE" = "transparent")
      )
    }
  }

  return(p)
}

plotQC <- function(...) {
  .Deprecated("plotQCmetrics")
  plotQCmetrics(...)
}

#' Flag Visium Outliers in SpatialExperiment Objects
#'
#' The `flagVisiumoutliers` function identifies and flags Visium systematic outlier spots in a
#' `SpatialExperiment` object based on barcodes. These outliers are marked in the `colData`
#' of the `SpatialExperiment` object, allowing users to exclude them from downstream analyses
#' to enhance data quality and reliability.
#'
#' @param spe A `SpatialExperiment` object containing spatial transcriptomics data.
#'   The object must include `array_row` and `array_col` columns in its `colData` that
#'   specify the spatial coordinates of each spot.
#'
#' @return A `SpatialExperiment` object identical to the input `spe` but with an additional
#'   logical column `systematic_outliers` in its `colData`. This column indicates whether
#'   each spot is flagged as a technical outlier (`TRUE`) or not (`FALSE`).
#'
#' @importFrom utils data
#' @import SpatialExperiment
#'
#' @export
#'
#' @examples
#' library(SpotSweeper)
#' library(SpatialExperiment)
#'
#' # load example data
#' spe <- STexampleData::Visium_humanDLPFC()
#'
#' # Flag outlier spots
#' spe <- flagVisiumOutliers(spe)
#'
#' # drop outlier spots
#' spe <- spe[, !colData(spe)$systematic_outliers]
#'
flagVisiumOutliers <- function(spe) {

  # Check if 'spe' is (or inherits from) SpatialExperiment
  if (!inherits(spe, "SpatialExperiment")) {
    stop("Input data must be a SpatialExperiment or inherit from SpatialExperiment.")
  }

  # Load the biased_spots dataset
  data("biased_spots", package = "SpotSweeper", envir = environment())

  # Create a logical mask for cells to drop
  drop_mask <- rep(FALSE, ncol(spe))  # Start with a mask that keeps everything

  # For each pair of (row, col) in biased_spots, mark the matching cells for removal
  for (i in 1:nrow(biased_spots)) {
    row_match <- spe$array_row == biased_spots$row[i]
    col_match <- spe$array_col == biased_spots$col[i]

    # Mark cells for removal where both conditions are true
    drop_mask <- drop_mask | (row_match & col_match)
  }

  # Add the drop_mask as a new column in colData
  colData(spe)$systematic_outliers <- drop_mask

  return(spe)
}

#' localVariance Function
#'
#' This function calculates the local variance based on kNN.
#'
#' @param spe SpatialExperiment object with the following columns in colData:
#'        sample_id, sum_umi, sum_gene
#' @param n_neighbors Number of nearest neighbors to use for variance
#'        calculation
#' @param metric Metric to use for variance calculation
#' @param samples Column in colData to use for sample ID
#' @param log Whether to log1p transform the metric
#' @param name Name of the new column to add to colData
#' @param workers Number of workers to use for parallel computation
#'
#' @return SpatialExperiment object with metric variance added to colData
#'
#' @importFrom SummarizedExperiment colData
#' @importFrom BiocNeighbors findKNN
#' @importFrom BiocParallel MulticoreParam
#' @importFrom MASS rlm
#'
#' @export localVariance
#'
#' @examples
#'
#' # for more details see extended example in vignettes
#' library(SpotSweeper)
#' library(SpatialExperiment)
#' library(escheR)
#'
#' # load example data
#' spe <- STexampleData::Visium_humanDLPFC()
#'
#' # change from gene id to gene names
#' rownames(spe) <- rowData(spe)$gene_name
#'
#' # show column data before SpotSweepR
#' colnames(colData(spe))
#'
#' # drop out-of-tissue spots
#' spe <- spe[, spe$in_tissue == 1]
#' spe <- spe[, !is.na(spe$ground_truth)]
#'
#' # Identifying the mitochondrial transcripts in our SpatialExperiment.
#' is.mito <- rownames(spe)[grepl("^MT-", rownames(spe))]
#'
#' # Calculating QC metric for each spot using scuttle
#' spe <- scuttle::addPerCellQCMetrics(spe, subsets = list(Mito = is.mito))
#' colnames(colData(spe))
#'
#' spe <- localVariance(spe,
#'     metric = "subsets_Mito_percent",
#'     n_neighbors = 36,
#'     name = "local_mito_variance_k36",
#'     workers = 1
#'     )
#'
#'
localVariance <- function(spe, n_neighbors = 36,
                          metric = c("expr_chrM_ratio"),
                          samples = "sample_id", log = FALSE, name = NULL,
                          workers = 1) {

  # ===== Validity checks =====
  # Check if 'spe' is (or inherits from) SpatialExperiment
  if (!inherits(spe, "SpatialExperiment")) {
    stop("Input data must be a SpatialExperiment or inherit from SpatialExperiment.")
  }

  # Validate 'metric' is a character vector
  if (!is.character(metric)) {
    stop("'metric' must be a character vector.")
  }

  # validate 'metric' are valid colData
  if (!all(metric %in% colnames(colData(spe)))) {
    stop("Metric must be present in colData.")
  }

  # validate samples is valid colData
  if (!samples %in% colnames(colData(spe))) {
    stop("Samples column must be present in colData.")
  }

  # Check 'n_neighbors' is a positive integer
  if (!is.numeric(n_neighbors) ||
      n_neighbors <= 0 ||
      n_neighbors != round(n_neighbors)) {
    stop("'n_neighbors' must be a positive integer.")
  }

  # Check 'workers' is a positive integer
  if (!is.numeric(workers) ||
      workers <= 0 ||
      workers != round(workers)) {
    stop("'workers' must be a positive integer.")
  }

  # ===== start function =====
  # log1p transform specified metric
  if (log) {
    metric_log <- paste0(metric, "_log")
    colData(spe)[metric_log] <- log1p(colData(spe)[[metric]])
    metric_to_use <- metric_log
  } else {
    metric_to_use <- metric
  }

  # Get a list of unique sample IDs
  unique_sample_ids <- unique(colData(spe)[[samples]])

  # Initialize list to store each columnData dataframe
  columnData_list <- sapply(unique_sample_ids, FUN = function(x) NULL)

  # Loop through each unique sample ID
  for (sample_id in seq_along(unique_sample_ids)) {
    # Subset the data for the current sample
    sample <- unique_sample_ids[sample_id]
    spe_subset <- subset(spe, , sample_id == sample)

    # Create a list of spatial coordinates and qc metric
    columnData <- colData(spe_subset)
    columnData$coords <- spatialCoords(spe_subset)

    # Find nearest neighbors
    dnn <- BiocNeighbors::findKNN(spatialCoords(spe_subset),
                                  k = n_neighbors,
                                  BPPARAM = BiocParallel::MulticoreParam(workers),
                                  warn.ties = FALSE
    )$index

    #  === Compute local variance ===
    # get neighborhood metric
    neighborhoods <- lapply(seq_len(nrow(dnn)), function(i) {
      indices <- dnn[i, ]
      indices <- indices[indices != 0]
      indices <- c(i, indices)

      columnData[indices, metric_to_use]
    })

    # Compute variance and mean
    stats_matrix <- t(sapply(neighborhoods, function(x) {
      c(var = var(x, na.rm = TRUE), mean = mean(x, na.rm = TRUE))
    }))

    # Handle non-finite values
    stats_matrix[!is.finite(stats_matrix)] <- 0

    # Perform robust linear regression to regress out mean-var bias
    fit.irls <- MASS::rlm(log2(var) ~ mean,
                          data = as.data.frame(stats_matrix))

    var_resid <- resid(fit.irls) # Get residuals

    # add local variance to columnData dataframe
    if (!is.null(name)) {
      columnData[name] <- var_resid
    } else {
      metric_var <- paste0(metric, "_var")
      columnData[metric_var] <- var_resid
    }

    # Store the modified columnData dataframe in the list
    columnData_list[[sample_id]] <- columnData
  }

  # rbind the list of dataframes
  columnData_aggregated <- do.call(rbind, columnData_list)

  # replace SPE column data with aggregated data
  colData(spe) <- columnData_aggregated

  return(spe)
}

#' Identify and annotate artifacts in spatial transcriptomics data
#'
#' This function identifies and annotates potential artifacts in spatial
#' transcriptomics data. Artifacts are detected based on local mito variance,
#' and the results are added to the original
#' SpatialExperiment (sce) object.
#'
#' @param spe A SingleCellExperiment object.
#' @param mito_percent The column name representing the mitochondrial percent.
#' Default is 'expr_chrM_ratio'.
#' @param mito_sum The column name representing sum mitochondrial expression.
#' Default is 'expr_chrM'.
#' @param samples The column name representing sample IDs. Default is
#' 'sample_id'.
#' @param n_order The number of orders for local mito variance calculation.
#' Default is 5.
#' @param shape The shape of the neighborhood for local variance calculation.
#' Can be either 'hexagonal' or 'square'. Default is 'hexagonal'.
#' @param log Logical, indicating whether to log1p transform mito_percent.
#' Default is TRUE.
#' @param name Prefix for the local variance column names. Default is
#' 'artifact'.
#' @param var_output Logical, indicating whether to include local variances in
#' the output. Default is TRUE.
#'
#' @return Returns the modified SingleCellExperiment object with artifact
#' annotations.
#'
#' @seealso
#' \code{\link{localVariance}}
#'
#' @import SingleCellExperiment
#' @import SpatialExperiment
#' @importFrom SummarizedExperiment colData
#' @importFrom stats prcomp kmeans resid var
#'
#'
#' @examples
#' library(SpotSweeper)
#' library(SpatialExperiment)
#' library(escheR)
#'
#' data(DLPFC_artifact)
#' spe <- DLPFC_artifact
#'
#' # find artifacts
#' spe <- findArtifacts(spe,
#'     mito_percent = "expr_chrM_ratio",
#'     mito_sum = "expr_chrM",
#'     n_order = 2, # 5 recommended, using 2 for time
#'     shape = "hexagonal",
#'     name = "artifact"
#' )
#'
#'
#' @export
findArtifacts <- function(
    spe, mito_percent = "expr_chrM_ratio",
    mito_sum = "expr_chrM", samples = "sample_id", n_order = 5,
    shape = "hexagonal", log = TRUE, name = "artifact", var_output = TRUE) {

  # ===== Validity checks =====
  # Check if 'spe' is (or inherits from) SpatialExperiment
  if (!inherits(spe, "SpatialExperiment")) {
    stop("Input data must be a SpatialExperiment or inherit from SpatialExperiment.")
  }

  if (!all(mito_percent %in% colnames(colData(spe)))) {
    stop("mito_percent must be present in colData.")
  }

  if (!mito_sum %in% colnames(colData(spe))) {
    stop("mito_sum must be present in colData.")
  }

  if (!samples %in% colnames(colData(spe))) {
    stop("Samples column must be present in colData.")
  }

  if (!is.numeric(n_order) ||
      n_order <= 0 ||
      n_order != round(n_order)) {
    stop("'n_order' must be a positive integer.")
  }

  if (!shape %in% c("hexagonal", "square")) {
    stop("'shape' must be either 'hexagonal' or 'square'.")
  }

  # Check if spe contains more than one sample_id
  unique_sample_ids <- unique(colData(spe)[[samples]])
  if (length(unique_sample_ids) > 1) {
    stop("The input SPE must contain only one sample. Please ensure only samples containing artifacts are passed.")
  }

  # Initialize a list to store spe for each sample
  columnData_list <- sapply(unique_sample_ids, FUN = function(x) NULL)

  for (sample in unique_sample_ids) {
    # subset by sample
    spe.temp <- spe[, colData(spe)[[samples]] == sample]

    # ======= Calculate local mito variance ========
    # Use vapply to iterate over order_seq
    var_matrix <- vapply(seq_len(n_order), function(i) {
      # Calculate n_neighbors for the current order
      if (shape == "hexagonal") {
        n_neighbors <- 3 * i * (i + 1)
      } else if (shape == "square") {
        n_neighbors <- 4 * i * (i + 1)
      }
      tmp.name <- paste0("k", n_neighbors)

      # Apply local variance calculation
      spe.temp <<- localVariance(spe.temp,
                                 metric = mito_percent,
                                 n_neighbors = n_neighbors,
                                 name = tmp.name,
                                 log=log)

      # Extract and return the column corresponding to tmp.name from colData
      colData(spe.temp)[[tmp.name]]
    }, numeric(length(spe.temp[[1]])))


    # ========== PCA and clustering ==========
    var_matrix <- cbind(
      var_matrix,

      colData(spe.temp)[[mito_percent]],
      colData(spe.temp)[[mito_sum]]
    )

    var_df <- data.frame(var_matrix)

    # Run PCA and add to reduced dims
    pc <- prcomp(var_df, center = TRUE, scale. = TRUE)
    rownames(pc$x) <- colnames(spe.temp) # assign rownames to avoid error
    reducedDim(spe.temp, "PCA_artifacts") <- pc$x

    # Cluster using kmeans and add to temp sce
    clus <- kmeans(pc$x, centers = 2, nstart = 25)
    spe.temp$Kmeans <- clus$cluster

    # =========== Artifact annotation ===========

    # calculate average local variance of the two clusters
    clus1_mean <- mean(colData(spe.temp)[[paste0("k", if (shape == "hexagonal") 18 else 24)]][spe.temp$Kmeans == 1])
    clus2_mean <- mean(colData(spe.temp)[[paste0("k", if (shape == "hexagonal") 18 else 24)]][spe.temp$Kmeans == 2])

    artifact_clus <- which.min(c(clus1_mean, clus2_mean))

    # create a new $artifact column; if clus1 mean < clus2 - rename Kmeans
    # 1 to 'TRUE'
    spe.temp$artifact <- FALSE
    spe.temp$artifact[spe.temp$Kmeans == artifact_clus] <- TRUE
    spe.temp$Kmeans <- NULL

    # add to sample list
    columnData_list[[sample]] <- colData(spe.temp)
  }

  # rbind the list of dataframes
  columnData_aggregated <- do.call(rbind, columnData_list)

  # replace SPE column data with aggregated data
  colData(spe) <- columnData_aggregated
  reducedDims(spe) <- reducedDims(spe.temp)

  # return sce
  return(spe)
}

# run examples from findArtifacts() function documentation
data(DLPFC_artifact)
spe <- DLPFC_artifact

# find artifacts using
set.seed(123)
spe <- findArtifacts(spe,
                     mito_percent = "expr_chrM_ratio",
                     mito_sum = "expr_chrM",
                     n_order = 2,
                     name = "artifact"
)


# === Tests ===
test_that("example objects have correct class", {
  expect_s4_class(spe, "SpatialExperiment")
})

test_that("examples give correct number of colData", {
  expect_equal(length(colnames(colData(spe))), 23)
})


# NOTE: this test currently gives essentially the same, but not exact numbers.
# this likely has to do with using exact knn ties being chosen randomly in edge
# cases.
#
# This is very minor as findArtifacts test always passes with the same number
# of artifact points, so the function is working as expected.
#
# test_that("examples gives correct local variance", {
#  expect_equal), c(0.6205693,  0.8147886,  0.6324888,
#                                         -0.9733463,  1.0527102,  0.8142039,
#                                         -2.9437993, -0.9599085,  0.2998882,
#                                         0.9236396))
# })

# run examples from findArtifacts() function documentation
data(DLPFC_artifact)
spe <- DLPFC_artifact

# find artifacts using
set.seed(123)
spe <- findArtifacts(spe,
                     mito_percent = "expr_chrM_ratio",
                     mito_sum = "expr_chrM",
                     n_order = 2,
                     name = "artifact"
)

# === Tests ===
test_that("example objects have correct class", {
  expect_s4_class(spe, "SpatialExperiment")
})

test_that("example object contains artifacts colDta", {
  expect_equal(dim(spe), c(5000, 3529))
})

# test_that("examples gives correct number of artifact spots", {
#   expect_equal(sum(spe$artifact), 517)
# })

#' human DLPFC dataset with a technical artifact (hangnail).
#'
#' The `DLPFC_artifact` dataset is a `SpatialExperiment` object containing a
#' single-sample subset of the human dorsolateral prefrontal cortex (DLPFC)
#' dataset from Hukki-Myers et al. 2023. This particular sample ('Br2743_ant')
#' is included to demonstrate the identification and removal of technical
#' artifacts within spatial transcriptomics data. The dataset serves as an
#' example for artifact detection using the 'SpotSweeper' workflow.
#'
#'
#' @docType data
#'
#' @usage data(DLPFC_artifact)
#'
#' @format An SpatialExperiment object.
#'
#' @keywords datasets
#'
#' @references Hukki-Myers et al. (2023) bioRxiv
#' (\href{https://www.biorxiv.org/content/10.1101/2023.02.15.528722v1}{bioRxiv})
#'
#' @source \href{https://github.com/LieberInstitute/spatialDLPFC}{spatialLIBD}
#'
#' @examples
#' data(DLPFC_artifact)
"DLPFC_artifact"

#' localOutliers Function
#'
#' This function detects local outliers in spatial transcriptomics data based on
#' standard quality control metrics, such as library size, unique genes, and
#' mitochondrial ratio. Local outliers are defined as spots with low/high
#' quality metrics compared to their surrounding neighbors, based on a modified
#' z-score statistic.
#'
#' @param spe SpatialExperiment or SingleCellExperiment object
#' @param metric colData QC metric to use for outlier detection
#' @param direction Direction of outlier detection (higher, lower, or both)
#' @param n_neighbors Number of nearest neighbors to use for outlier detection
#' @param samples Column name in colData to use for sample IDs
#' @param log Logical indicating whether to log1p transform the features
#' (default is TRUE)
#' @param cutoff Cutoff for outlier detection (default is 3)
#' @param workers Number of workers for parallel processing (default is 1)
#'
#' @return SpatialExperiment or SingleCellExperiment object with updated colData containing outputs
#'
#' @importFrom SummarizedExperiment colData
#' @importFrom BiocNeighbors findKNN
#' @importFrom spatialEco outliers
#' @importFrom BiocParallel MulticoreParam
#'
#' @export localOutliers
#'
#' @examples
#' library(SpotSweeper)
#' library(SpatialExperiment)
#'
#' # load example data
#' spe <- STexampleData::Visium_humanDLPFC()
#'
#' # change from gene id to gene names
#' rownames(spe) <- rowData(spe)$gene_name
#'
#' # drop out-of-tissue spots
#' spe <- spe[, spe$in_tissue == 1]
#' spe <- spe[, !is.na(spe$ground_truth)]
#'
#' # Identifying the mitochondrial transcripts in our SpatialExperiment.
#' is.mito <- rownames(spe)[grepl("^MT-", rownames(spe))]
#'
#' # Calculating QC metrics for each spot using scuttle
#' spe <- scuttle::addPerCellQCMetrics(spe, subsets = list(Mito = is.mito))
#' colnames(colData(spe))
#'
#' # Identifying local outliers using SpotSweeper
#' spe <- localOutliers(spe,
#'                      metric = "sum",
#'                      direction = "lower",
#'                      log = TRUE
#' )
#'
localOutliers <- function(
    spe, metric = "detected",
    direction = "lower", n_neighbors = 36, samples = "sample_id",
    log = TRUE, cutoff = 3, workers=1) {

  # ===== Validity checks =====
  # Check if 'spe' is (or inherits from) SpatialExperiment
  if (!inherits(spe, "SpatialExperiment")) {
    stop("Input data must be a SpatialExperiment or inherit from SpatialExperiment.")
  }

  # Validate 'direction'
  if (!direction %in% c("lower", "higher", "both")) {
    stop("'direction' must be one of 'lower', 'higher', or 'both'.")
  }

  # Check 'n_neighbors' is a positive integer
  if (!is.numeric(n_neighbors) ||
      n_neighbors <= 0 ||
      n_neighbors != round(n_neighbors)) {
    stop("'n_neighbors' must be a positive integer.")
  }

  # Check 'cutoff' is a numeric value
  if (!is.numeric(cutoff)) {
    stop("'cutoff' must be a numeric value.")
  }

  # ===== Start function =====
  # log transform specified metric
  if (log) {
    metric_log <- paste0(metric, "_log")
    colData(spe)[metric_log] <- log1p(colData(spe)[[metric]])
    metric_to_use <- metric_log
  } else {
    metric_to_use <- metric
  }

  # Get a list of unique sample IDs
  unique_sample_ids <- unique(colData(spe)[[samples]])

  # Initialize list to store each columnData
  columnData_list <- sapply(unique_sample_ids, FUN = function(x) NULL)

  # Loop through each unique sample ID
  for (sample in unique_sample_ids) {
    # Subset the data for the current sample
    spe_subset <- spe[, colData(spe)[[samples]] == sample]

    # Create a list of spatial coordinates and qc features
    columnData <- colData(spe_subset)

    # Find nearest neighbors
    dnn <- BiocNeighbors::findKNN(spatialCoords(spe_subset),
                                  k = n_neighbors, warn.ties = FALSE,
                                  BPPARAM = BiocParallel::MulticoreParam(workers=workers))$index

    # get neighborhood metrics
    neighborhoods <- lapply(seq_len(nrow(dnn)), function(i) {
      indices <- dnn[i, ]
      indices <- indices[indices != 0]
      indices <- c(i, indices)

      columnData[indices, metric_to_use]
    })

    # Compute modified-z and return the middle spot
    mod_z_matrix <- vapply(neighborhoods, function(x) {
      spatialEco::outliers(x)[1]
    }, numeric(1))

    # Handle non-finite values
    mod_z_matrix[!is.finite(mod_z_matrix)] <- 0

    # find outliers based on cutoff, store in colData
    metric_outliers <- paste0(metric, "_outliers")
    columnData[metric_outliers] <- switch(direction,
                                          higher = sapply(
                                            mod_z_matrix,
                                            function(x) x > cutoff
                                          ),
                                          lower = sapply(
                                            mod_z_matrix,
                                            function(x) x < -cutoff
                                          ),
                                          both = sapply(
                                            mod_z_matrix,
                                            function(x) {
                                              x > cutoff | x <
                                                -cutoff
                                            }
                                          )
    )

    # add z-scores to colData
    metric_z <- paste0(metric, "_z")
    columnData[metric_z] <- mod_z_matrix[]

    # Store the modified columnData dataframe in the list
    columnData_list[[sample]] <- columnData
  }

  # rbind the list of dataframes
  columnData_aggregated <- do.call(rbind, columnData_list)

  # replace column data
  colData(spe) <- columnData_aggregated

  return(spe)
}

# run examples from localOutliers() function documentation
library(SpatialExperiment)

# load example data
spe <- STexampleData::Visium_humanDLPFC()

# change from gene id to gene names
rownames(spe) <- rowData(spe)$gene_name

# drop out-of-tissue spots
spe <- spe[, spe$in_tissue == 1]
spe <- spe[, !is.na(spe$ground_truth)]

# Identifying the mitochondrial transcripts in our SpatialExperiment.
is.mito <- rownames(spe)[grepl("^MT-", rownames(spe))]

# Calculating QC metrics for each spot using scuttle
spe <- scuttle::addPerCellQCMetrics(spe, subsets = list(Mito = is.mito))
colnames(colData(spe))

# Identifying local outliers using SpotSweeper
spe <- localOutliers(spe,
                     metric = "sum",
                     direction = "lower",
                     log = TRUE
)

# === Tests ===
test_that("example objects have correct class", {
  expect_s4_class(spe, "SpatialExperiment")
})

test_that("correct outliers were found", {
  expect_equal(sum(as.logical(spe$sum_outliers)), 7)
})

#' Biased Spots Data
#'
#' The `biased_spots` dataset is a `data.frame` containing information about specific
#' spatial spots identified as technical outliers in spatial transcriptomics experiments.
#' Each entry represents a biased spot characterized by its spatial coordinates
#' (row and column) and a unique barcode. This dataset is utilized by the
#' `flagVisiumOutliers` function to flag and exclude these outlier spots from
#' downstream analyses, thereby enhancing data quality and reliability.
#'
#' @docType data
#'
#' @usage data(biased_spots)
#'
#' @format A `data.frame` with the following columns:
#' \describe{
#'   \item{row}{Integer. The row position of a biased spot within the spatial grid.}
#'   \item{col}{Integer. The column position of a biased spot within the spatial grid.}
#'   \item{barcode}{Character. A unique identifier corresponding to the spatial transcriptomics barcode of the biased spot.}
#' }
#'
#' @keywords datasets
#'
#' @source
#' The `biased_spots.rds` file was generated in the analysis of local outliers.
#' See https://github.com/boyiguo1/Manuscript-SpotSweeper/blob/main/code/03_Visium/figure_3.R for more details.
#'
#'
#' @examples
#' data(biased_spots)
"biased_spots"

#' Plot Outlier Metrics to PDF
#'
#' This function generates a PDF file containing plots for each sample in the
#' SpatialExperiment object, highlighting outliers based on specified metrics.
#' Each plot visualizes outlier metrics for a single sample, allowing for
#' easy comparison and analysis across samples.
#'
#' @param spe A SpatialExperiment object containing the data to be plotted.
#' @param sample_id A character string specifying the column name in
#'   `colData(spe)` that contains unique sample identifiers. Default is
#'   'sample_id'.
#' @param metric A character string specifying the metric to be visualized
#'   in the plot. This metric should be a column name in `colData(spe)`.
#' @param outliers A character string specifying the column name in
#'   `colData(spe)` that indicates whether a data point is considered an
#'   outlier. Default is local_outliers'.
#' @param colors A character vector specifying the colors to be used for the
#'  gradient scale. If length is 2, the gradient will be a single color gradient
#' @param stroke A numeric value specifying the border thickness for outlier
#'   points. Default is 1.
#' @param width A numeric value indicating the width of the plot. Default
#'   is 5.
#' @param height A numeric value indicating the height of the plot. Default
#'   is 5.
#' @param fname A character string specifying the path and name of the output
#'  PDF file.
#' @param point_size A numeric value specifying the size of the points in the
#'  plot. Default is 2.
#'
#' @return ggplot object if specified. Generates a plot otherwise.
#'
#' @importFrom escheR make_escheR add_fill add_ground
#' @importFrom grDevices dev.off pdf
#'
#' @examples
#' library(SpotSweeper)
#' library(SpatialExperiment)
#' library(escheR)
#'
#' # load example data
#' spe <- STexampleData::Visium_humanDLPFC()
#'
#' tempFilePath <- file.path(tempdir(), "examplePlot.pdf")
#'
#' # change from gene id to gene names
#' rownames(spe) <- rowData(spe)$gene_name
#'
#' # drop out-of-tissue spots
#' spe <- spe[, spe$in_tissue == 1]
#' spe <- spe[, !is.na(spe$ground_truth)]
#'
#' # Identifying the mitochondrial transcripts in our SpatialExperiment.
#' is.mito <- rownames(spe)[grepl("^MT-", rownames(spe))]
#'
#' # Calculating QC metrics for each spot using scuttle
#' spe <- scuttle::addPerCellQCMetrics(spe, subsets = list(Mito = is.mito))
#' colnames(colData(spe))
#'
#' # Identifying local outliers using SpotSweeper
#' spe <- localOutliers(spe,
#'                      metric = "sum",
#'                      direction = "lower",
#'                      log = TRUE
#' )
#'
#' plotQCpdf(spe,
#'           metric="sum",
#'           outliers="sum_outliers",
#'           fname=tempFilePath)
#'
#' @export
plotQCpdf <- function(
        spe, sample_id = "sample_id", metric = "detected",
        outliers = "local_outliers", colors = c("white", "black"), stroke = 1,
        point_size = 2, width = 5, height = 5, fname) {
    # Get a list of unique sample IDs
    unique_sample_ids <- unique(colData(spe)[[sample_id]])

    # initialize PDF and loop through plots
    pdf(width = width, height = height, fname)
    for (sample in unique_sample_ids) {
        p <- plotQC(spe,
            sample_id = sample_id, sample = sample,
            metric = metric, outliers = outliers, colors = colors,
            stroke = stroke, point_size = point_size
        )

        # print
        print(p)
    }
    dev.off()
}

---
title: "Getting started with 'SpotSweeper'"
author: 
  - name: Michael Totty
    affiliation: &id1 "Johns Hopkins Bloomberg School of Public Health, 
    Baltimore, MD, USA"
  - name: Boyi Guo
    affiliation: *id1
  - name: Stephanie Hicks
    affiliation: *id1
date: "`r Sys.Date()`"
output: BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{Getting Started with `SpotSweeper`}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```




## Introduction

`SpotSweeper` is an R package for spatial transcriptomics data quality control 
(QC). It provides functions for detecting and visualizing spot-level local 
outliers and artifacts using spatially-aware methods. The package is designed 
to work with [SpatialExperiment](https://github.com/drighelli/SpatialExperiment)
objects, and is compatible with data from 10X Genomics Visium and other spatial 
transcriptomics platforms.

## Installation

Currently, the only way to install `SpotSweeper` is by downloading the 
development version which can be installed from 
[GitHub](https://github.com/MicTott/SpotSweeper) using the following: 

```{r 'install_dev', eval = FALSE}
if (!require("devtools")) install.packages("devtools")
remotes::install_github("MicTott/SpotSweeper")
```

Once accepted in [Bioconductor](http://bioconductor.org/), `SpotSweeper` will be
installable using:

```{r 'install', eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("SpotSweeper")
```

## Spot-level local outlier detection

### Loading example data

Here we'll walk you through the standard workflow for using 'SpotSweeper' to 
detect and visualize local outliers in spatial transcriptomics data. We'll use 
the `Visium_humanDLPFC` dataset from the `STexampleData` package, which is a 
`SpatialExperiment` object.

Because local outliers will be saved in the `colData` of the `SpatialExperiment`
object, we'll first view the `colData` and drop out-of-tissue spots before 
calculating quality control (QC) metrics and running `SpotSweeper`. 


```{r example_spe}
library(SpotSweeper)

# load  Maynard et al DLPFC daatset
spe <- STexampleData::Visium_humanDLPFC()

# show column data before SpotSweeper
colnames(colData(spe))

# drop out-of-tissue spots
spe <- spe[, spe$in_tissue == 1]
```

### Calculating QC metrics using `scuttle`

We'll use the `scuttle` package to calculate QC metrics. To do this, we'll need 
to first change the `rownames` from gene id to gene names. We'll then get the 
mitochondrial transcripts and calculate QC metrics for each spot using 
`scuttle::addPerCellQCMetrics`.

```{r example_scuttle}
# change from gene id to gene names
rownames(spe) <- rowData(spe)$gene_name

# identifying the mitochondrial transcripts
is.mito <- rownames(spe)[grepl("^MT-", rownames(spe))]

# calculating QC metrics for each spot using scuttle
spe <- scuttle::addPerCellQCMetrics(spe, subsets = list(Mito = is.mito))
colnames(colData(spe))
```


### Identifying local outliers using `SpotSweeper`


We can now use `SpotSweeper` to identify local outliers in the spatial 
transcriptomics data. We'll use the `localOutliers` function to detect local 
outliers based on the unique detected genes, total library size, and percent of 
the total reads that are mitochondrial. These methods assume a normal 
distribution, so we'll use the log-transformed sum of the counts and the 
log-transformed number of detected genes. For mitochondrial percent, we'll use 
the raw mitochondrial percentage. 

```{r example_local_outliers}
# library size
spe <- localOutliers(spe,
    metric = "sum",
    direction = "lower",
    log = TRUE
)

# unique genes
spe <- localOutliers(spe,
    metric = "detected",
    direction = "lower",
    log = TRUE
)

# mitochondrial percent
spe <- localOutliers(spe,
    metric = "subsets_Mito_percent",
    direction = "higher",
    log = FALSE
)
```

The `localOutlier` function automatically outputs the results to the `colData` 
with the naming convention `X_outliers`, where `X` is the name of the input 
`colData`. We can then combine all outliers into a single column called 
`local_outliers` in the `colData` of the `SpatialExperiment` object.

```{r example_combine_local_outliers}
# combine all outliers into "local_outliers" column
spe$local_outliers <- as.logical(spe$sum_outliers) |
    as.logical(spe$detected_outliers) |
    as.logical(spe$subsets_Mito_percent_outliers)
```

### Visualizing local outliers

We can visualize the local outliers using the `plotQCmetrics` function. This 
function creates a scatter plot of the specified metric and highlights the 
local outliers in red using the `escheR` package. Here, we'll visualize local 
outliers of library size, unique genes, mitochondrial percent, and finally, all
local outliers. We'll then arrange these plots in a grid using 
`ggpubr::arrange`.

```{r local_outlier_plot, fig.width=3, fig.height=3, dpi=100}
library(escheR)


# all local outliers
plotQCmetrics(spe, metric = "sum_log", outliers = "local_outliers", point_size = 1.1, 
       stroke = 0.75) +
      ggtitle("All Local Outliers")

```


## Removing technical artifacts using `SpotSweeper`

### Loading example data

```{r example_artifactRemoval}
# load in DLPFC sample with hangnail artifact
data(DLPFC_artifact)
spe <- DLPFC_artifact

# inspect colData before artifact detection
colnames(colData(spe))
```


### Visualizing technical artifacts

Technical artifacts can commonly be visualized by standard QC metrics, including
library size, unique genes, or mitochondrial percentage. We can first visualize
the technical artifacts using the `plotQCmetrics` function. In this sample, we can 
clearly see a hangnail artifact on the right side of the tissue section
in the mitochondrial ratio plot.

```{r artifact_QC_plots, fig.width=4, fig.height=4, dpi=100}

plotQCmetrics(spe,
    metric = "expr_chrM_ratio",
    outliers = NULL, point_size = 1.1
) +
    ggtitle("Mitochondrial Percent")

```

### Identifying artifacts using `SpotSweeper`

We can then use the `findArtifacts` function to identify artifacts in the 
spatial transcriptomics (data. This function identifies technical artifacts 
based on the first principle component of the local variance of the specified QC
metric (`mito_percent`) at numerous neighorhood sizes (`n_order=5`). Currently,
`kmeans` clustering is used to cluster the technical artifact vs high-quality 
Visium spots. Similar to `localOutliers`, the `findArtifacts` function then
outputs the results to the `colData`.

```{r artifact_plot}
# find artifacts using SpotSweeper
spe <- findArtifacts(spe,
    mito_percent = "expr_chrM_ratio",
    mito_sum = "expr_chrM",
    n_order = 5,
    name = "artifact"
)

# check that "artifact" is now in colData
colnames(colData(spe))
```


### Visualizing artifacts

We can visualize the artifacts using the `escheR` package. Here, we'll visualize
the artifacts using the `plotQCmetrics` function and arrange these plots using 
`ggpubr::arrange`.

```{r artifact_visualization, fig.width=4, fig.height=4, dpi=100}
plotQCmetrics(spe,
    metric = "expr_chrM_ratio",
    outliers = "artifact", point_size = 1.1
) +
    ggtitle("Hangnail artifact")
```
# Session information

```{r}
utils::sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biased_spots.R
\docType{data}
\name{biased_spots}
\alias{biased_spots}
\title{Biased Spots Data}
\format{
A \code{data.frame} with the following columns:
\describe{
\item{row}{Integer. The row position of a biased spot within the spatial grid.}
\item{col}{Integer. The column position of a biased spot within the spatial grid.}
\item{barcode}{Character. A unique identifier corresponding to the spatial transcriptomics barcode of the biased spot.}
}
}
\source{
The \code{biased_spots.rds} file was generated in the analysis of local outliers.
See https://github.com/boyiguo1/Manuscript-SpotSweeper/blob/main/code/03_Visium/figure_3.R for more details.
}
\usage{
data(biased_spots)
}
\description{
The \code{biased_spots} dataset is a \code{data.frame} containing information about specific
spatial spots identified as technical outliers in spatial transcriptomics experiments.
Each entry represents a biased spot characterized by its spatial coordinates
(row and column) and a unique barcode. This dataset is utilized by the
\code{flagVisiumOutliers} function to flag and exclude these outlier spots from
downstream analyses, thereby enhancing data quality and reliability.
}
\examples{
data(biased_spots)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/localVariance.R
\name{localVariance}
\alias{localVariance}
\title{localVariance Function}
\usage{
localVariance(
  spe,
  n_neighbors = 36,
  metric = c("expr_chrM_ratio"),
  samples = "sample_id",
  log = FALSE,
  name = NULL,
  workers = 1
)
}
\arguments{
\item{spe}{SpatialExperiment object with the following columns in colData:
sample_id, sum_umi, sum_gene}

\item{n_neighbors}{Number of nearest neighbors to use for variance
calculation}

\item{metric}{Metric to use for variance calculation}

\item{samples}{Column in colData to use for sample ID}

\item{log}{Whether to log1p transform the metric}

\item{name}{Name of the new column to add to colData}

\item{workers}{Number of workers to use for parallel computation}
}
\value{
SpatialExperiment object with metric variance added to colData
}
\description{
This function calculates the local variance based on kNN.
}
\examples{

# for more details see extended example in vignettes
library(SpotSweeper)
library(SpatialExperiment)
library(escheR)

# load example data
spe <- STexampleData::Visium_humanDLPFC()

# change from gene id to gene names
rownames(spe) <- rowData(spe)$gene_name

# show column data before SpotSweepR
colnames(colData(spe))

# drop out-of-tissue spots
spe <- spe[, spe$in_tissue == 1]
spe <- spe[, !is.na(spe$ground_truth)]

# Identifying the mitochondrial transcripts in our SpatialExperiment.
is.mito <- rownames(spe)[grepl("^MT-", rownames(spe))]

# Calculating QC metric for each spot using scuttle
spe <- scuttle::addPerCellQCMetrics(spe, subsets = list(Mito = is.mito))
colnames(colData(spe))

spe <- localVariance(spe,
    metric = "subsets_Mito_percent",
    n_neighbors = 36,
    name = "local_mito_variance_k36",
    workers = 1
    )


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/localOutliers.R
\name{localOutliers}
\alias{localOutliers}
\title{localOutliers Function}
\usage{
localOutliers(
  spe,
  metric = "detected",
  direction = "lower",
  n_neighbors = 36,
  samples = "sample_id",
  log = TRUE,
  cutoff = 3,
  workers = 1
)
}
\arguments{
\item{spe}{SpatialExperiment or SingleCellExperiment object}

\item{metric}{colData QC metric to use for outlier detection}

\item{direction}{Direction of outlier detection (higher, lower, or both)}

\item{n_neighbors}{Number of nearest neighbors to use for outlier detection}

\item{samples}{Column name in colData to use for sample IDs}

\item{log}{Logical indicating whether to log1p transform the features
(default is TRUE)}

\item{cutoff}{Cutoff for outlier detection (default is 3)}

\item{workers}{Number of workers for parallel processing (default is 1)}
}
\value{
SpatialExperiment or SingleCellExperiment object with updated colData containing outputs
}
\description{
This function detects local outliers in spatial transcriptomics data based on
standard quality control metrics, such as library size, unique genes, and
mitochondrial ratio. Local outliers are defined as spots with low/high
quality metrics compared to their surrounding neighbors, based on a modified
z-score statistic.
}
\examples{
library(SpotSweeper)
library(SpatialExperiment)

# load example data
spe <- STexampleData::Visium_humanDLPFC()

# change from gene id to gene names
rownames(spe) <- rowData(spe)$gene_name

# drop out-of-tissue spots
spe <- spe[, spe$in_tissue == 1]
spe <- spe[, !is.na(spe$ground_truth)]

# Identifying the mitochondrial transcripts in our SpatialExperiment.
is.mito <- rownames(spe)[grepl("^MT-", rownames(spe))]

# Calculating QC metrics for each spot using scuttle
spe <- scuttle::addPerCellQCMetrics(spe, subsets = list(Mito = is.mito))
colnames(colData(spe))

# Identifying local outliers using SpotSweeper
spe <- localOutliers(spe,
                     metric = "sum",
                     direction = "lower",
                     log = TRUE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotQCmetrics.R
\name{plotQCmetrics}
\alias{plotQCmetrics}
\title{Plot QC metrics for a Single Sample in a SpatialExperiment object}
\usage{
plotQCmetrics(
  spe,
  sample_id = "sample_id",
  sample = unique(spe$sample_id)[1],
  metric = "detected",
  outliers = NULL,
  point_size = 2,
  colors = c("white", "black"),
  stroke = 1
)
}
\arguments{
\item{spe}{A SpatialExperiment object containing the data to be plotted.}

\item{sample_id}{A character string specifying the column name in
\code{colData(spe)} that contains unique sample identifiers.
Default is "sample_id".}

\item{sample}{A character string or numeric value specifying the sample to be
plotted. By default, it plots the first unique sample found in
\code{spe$sample_id}.}

\item{metric}{A character string specifying the metric to be visualized
in the plot. This metric should be a column name in \code{colData(spe)}.}

\item{outliers}{A character string specifying the column name in
\code{colData(spe)} that indicates whether a data point is considered an
outlier. Default is NULL.}

\item{point_size}{A numeric value specifying the size of the points in the
plot. Default is 2.}

\item{colors}{A character vector specifying the colors to be used for the
gradient scale. If length is 2, the gradient will be a single color
gradient.}

\item{stroke}{A numeric value specifying the border thickness for outlier
points. Default is 1.}
}
\value{
The function returns a plot object created by \code{make_escheR} and
modified with additional layers for visualizing the specified metric and
outliers. The plot is not explicitly printed by the function and should be
printed by the caller.
}
\description{
This function generates a plot for a specified sample within a
SpatialExperiment object, highlighting outliers based on a specified
metric. The plot visualizes the metric of interest and indicates outliers
with a distinct color.
}
\examples{
library(SpotSweeper)
library(SpatialExperiment)
library(escheR)

# load example data
spe <- STexampleData::Visium_humanDLPFC()

# change from gene id to gene names
rownames(spe) <- rowData(spe)$gene_name

# drop out-of-tissue spots
spe <- spe[, spe$in_tissue == 1]
spe <- spe[, !is.na(spe$ground_truth)]

# Identifying the mitochondrial transcripts in our SpatialExperiment.
is.mito <- rownames(spe)[grepl("^MT-", rownames(spe))]

# Calculating QC metrics for each spot using scuttle
spe <- scuttle::addPerCellQCMetrics(spe, subsets = list(Mito = is.mito))
colnames(colData(spe))

# Identifying local outliers using SpotSweeper
spe <- localOutliers(spe,
                     metric = "sum",
                     direction = "lower",
                     log = TRUE
)

plotQCmetrics(spe, metric="sum", outliers="sum_outliers")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotQCpdf.R
\name{plotQCpdf}
\alias{plotQCpdf}
\title{Plot Outlier Metrics to PDF}
\usage{
plotQCpdf(
  spe,
  sample_id = "sample_id",
  metric = "detected",
  outliers = "local_outliers",
  colors = c("white", "black"),
  stroke = 1,
  point_size = 2,
  width = 5,
  height = 5,
  fname
)
}
\arguments{
\item{spe}{A SpatialExperiment object containing the data to be plotted.}

\item{sample_id}{A character string specifying the column name in
\code{colData(spe)} that contains unique sample identifiers. Default is
'sample_id'.}

\item{metric}{A character string specifying the metric to be visualized
in the plot. This metric should be a column name in \code{colData(spe)}.}

\item{outliers}{A character string specifying the column name in
\code{colData(spe)} that indicates whether a data point is considered an
outlier. Default is local_outliers'.}

\item{colors}{A character vector specifying the colors to be used for the
gradient scale. If length is 2, the gradient will be a single color gradient}

\item{stroke}{A numeric value specifying the border thickness for outlier
points. Default is 1.}

\item{point_size}{A numeric value specifying the size of the points in the
plot. Default is 2.}

\item{width}{A numeric value indicating the width of the plot. Default
is 5.}

\item{height}{A numeric value indicating the height of the plot. Default
is 5.}

\item{fname}{A character string specifying the path and name of the output
PDF file.}
}
\value{
ggplot object if specified. Generates a plot otherwise.
}
\description{
This function generates a PDF file containing plots for each sample in the
SpatialExperiment object, highlighting outliers based on specified metrics.
Each plot visualizes outlier metrics for a single sample, allowing for
easy comparison and analysis across samples.
}
\examples{
library(SpotSweeper)
library(SpatialExperiment)
library(escheR)

# load example data
spe <- STexampleData::Visium_humanDLPFC()

tempFilePath <- file.path(tempdir(), "examplePlot.pdf")

# change from gene id to gene names
rownames(spe) <- rowData(spe)$gene_name

# drop out-of-tissue spots
spe <- spe[, spe$in_tissue == 1]
spe <- spe[, !is.na(spe$ground_truth)]

# Identifying the mitochondrial transcripts in our SpatialExperiment.
is.mito <- rownames(spe)[grepl("^MT-", rownames(spe))]

# Calculating QC metrics for each spot using scuttle
spe <- scuttle::addPerCellQCMetrics(spe, subsets = list(Mito = is.mito))
colnames(colData(spe))

# Identifying local outliers using SpotSweeper
spe <- localOutliers(spe,
                     metric = "sum",
                     direction = "lower",
                     log = TRUE
)

plotQCpdf(spe,
          metric="sum",
          outliers="sum_outliers",
          fname=tempFilePath)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findArtifacts.R
\name{findArtifacts}
\alias{findArtifacts}
\title{Identify and annotate artifacts in spatial transcriptomics data}
\usage{
findArtifacts(
  spe,
  mito_percent = "expr_chrM_ratio",
  mito_sum = "expr_chrM",
  samples = "sample_id",
  n_order = 5,
  shape = "hexagonal",
  log = TRUE,
  name = "artifact",
  var_output = TRUE
)
}
\arguments{
\item{spe}{A SingleCellExperiment object.}

\item{mito_percent}{The column name representing the mitochondrial percent.
Default is 'expr_chrM_ratio'.}

\item{mito_sum}{The column name representing sum mitochondrial expression.
Default is 'expr_chrM'.}

\item{samples}{The column name representing sample IDs. Default is
'sample_id'.}

\item{n_order}{The number of orders for local mito variance calculation.
Default is 5.}

\item{shape}{The shape of the neighborhood for local variance calculation.
Can be either 'hexagonal' or 'square'. Default is 'hexagonal'.}

\item{log}{Logical, indicating whether to log1p transform mito_percent.
Default is TRUE.}

\item{name}{Prefix for the local variance column names. Default is
'artifact'.}

\item{var_output}{Logical, indicating whether to include local variances in
the output. Default is TRUE.}
}
\value{
Returns the modified SingleCellExperiment object with artifact
annotations.
}
\description{
This function identifies and annotates potential artifacts in spatial
transcriptomics data. Artifacts are detected based on local mito variance,
and the results are added to the original
SpatialExperiment (sce) object.
}
\examples{
library(SpotSweeper)
library(SpatialExperiment)
library(escheR)

data(DLPFC_artifact)
spe <- DLPFC_artifact

# find artifacts
spe <- findArtifacts(spe,
    mito_percent = "expr_chrM_ratio",
    mito_sum = "expr_chrM",
    n_order = 2, # 5 recommended, using 2 for time
    shape = "hexagonal",
    name = "artifact"
)


}
\seealso{
\code{\link{localVariance}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flagVisiumOutliers.R
\name{flagVisiumOutliers}
\alias{flagVisiumOutliers}
\title{Flag Visium Outliers in SpatialExperiment Objects}
\usage{
flagVisiumOutliers(spe)
}
\arguments{
\item{spe}{A \code{SpatialExperiment} object containing spatial transcriptomics data.
The object must include \code{array_row} and \code{array_col} columns in its \code{colData} that
specify the spatial coordinates of each spot.}
}
\value{
A \code{SpatialExperiment} object identical to the input \code{spe} but with an additional
logical column \code{systematic_outliers} in its \code{colData}. This column indicates whether
each spot is flagged as a technical outlier (\code{TRUE}) or not (\code{FALSE}).
}
\description{
The \code{flagVisiumoutliers} function identifies and flags Visium systematic outlier spots in a
\code{SpatialExperiment} object based on barcodes. These outliers are marked in the \code{colData}
of the \code{SpatialExperiment} object, allowing users to exclude them from downstream analyses
to enhance data quality and reliability.
}
\examples{
library(SpotSweeper)
library(SpatialExperiment)

# load example data
spe <- STexampleData::Visium_humanDLPFC()

# Flag outlier spots
spe <- flagVisiumOutliers(spe)

# drop outlier spots
spe <- spe[, !colData(spe)$systematic_outliers]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DLPFC_artifact.R
\docType{data}
\name{DLPFC_artifact}
\alias{DLPFC_artifact}
\title{human DLPFC dataset with a technical artifact (hangnail).}
\format{
An SpatialExperiment object.
}
\source{
\href{https://github.com/LieberInstitute/spatialDLPFC}{spatialLIBD}
}
\usage{
data(DLPFC_artifact)
}
\description{
The \code{DLPFC_artifact} dataset is a \code{SpatialExperiment} object containing a
single-sample subset of the human dorsolateral prefrontal cortex (DLPFC)
dataset from Hukki-Myers et al. 2023. This particular sample ('Br2743_ant')
is included to demonstrate the identification and removal of technical
artifacts within spatial transcriptomics data. The dataset serves as an
example for artifact detection using the 'SpotSweeper' workflow.
}
\examples{
data(DLPFC_artifact)
}
\references{
Hukki-Myers et al. (2023) bioRxiv
(\href{https://www.biorxiv.org/content/10.1101/2023.02.15.528722v1}{bioRxiv})
}
\keyword{datasets}
